
-module(link1).
-export([start/0, child_process/0]).

% shows 2-ways links, here the child process ends normally
% before the parent, and the parent continues executing to
% normal termination after child exits

% this uses default behavior, which is the a terminating
% process send a {EXIT, ... , normal }  signal to all
% linked processes, and also by default a normal signal
% is ignored by linked processes when received

start() ->
    % Parent process continues running after child finishes
    io:format("Parent>> process begins running.~n"),

    % Parent process spawns and links to the child process
    ChildPid = spawn_link(?MODULE, child_process, []),
    io:format("Parent>> process has spawned and linked to child.~n"),


    % Wait for a while before ending
    timer:sleep(4000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(4000),
    io:format("Parent>> process is still running.~n"),
    timer:sleep(4000),
    io:format("Parent>> process exiting normally.~n").

child_process() ->
    io:format("Child>> process is running.~n"),
    timer:sleep(2000),
    io:format("Child>> process is running.~n"),
    timer:sleep(2000),
    io:format("Child>> process is running.~n"),

    % Child process finishes normally
    io:format("Child process exiting normally.~n").

    % a normal exit signal is sent to parent, and since its normal
    % the parent ignores it and keep going

