
-module(hotS).
-compile([export_all]).

% usage: run hotS:hgo() to start the hot swap server
%    whereis(hs)            will gen the PID of this server
%    whereis(hs) ! SomeMsg  will send a message to this server 
%                           in the shell, and the server will 
%                           respond with print output
%
% do the same with hotS:cgo() to make a cold server
%
% now edit the hotS module code and change the output for 
%   each server
% now recompile the module into new .beam code
% now send messages to each server... 
%    the hot server will print out the new output format
%    the cold server will still print out the old format

% note that it takes two messages to see the hot swap... as the
% old server is still taking a message before it recurses and
% gets the new code

% the idea is we have two servers running, one using the 
% mod:fn() notation  so that is hot swap code when the module 
% is recompiled (the "hot" server)
%
% the other server (the "cold" server) is using func call noation 
% that is not qualified with the module name, and will not update 
% when the module is recompiled.. it will keep running old code it 
% ran when the server process was first started


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% hot server example
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hgo() ->
    Pid = spawn(hotS,server,[]),
    register(hs,Pid).  % hs hot server

server() -> receive
	       Msg ->
                  %io:fwrite("(Orig Hot Server) got msg: ~s~n", [Msg]),
                  io:fwrite("(Newer Hot Server) got msg: ~s~n", [Msg]),
                  hotS:server()  % module:func for hot update when recompiled
            end.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% cold server example
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cgo() ->
    Pid = spawn(hotS,cerver,[]),
    register(cs,Pid).  % cs cold server

cerver() -> receive
	       Msg ->
                  %io:fwrite("(Orig Cold Server) got msg: ~s~n", [Msg]),
                  io:fwrite("(Newer Cold Server) got msg: ~s~n", [Msg]),
                  cerver() % no code update when recompile the module
            end. 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% what about state?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

kgo() ->
    Pid = spawn(hotS,kserver,[0]),
    register(ks,Pid).  % ks kounting hot server

kserver(N) -> receive
	       Msg ->
                  io:fwrite("New Msg: ~s, Acc: ~p~n", [Msg, N+1]),
                  %io:fwrite("Newer Msg: ~s, Acc: ~p~n", [Msg, N+1]),
                  hotS:kserver(N+1)  % module:func for hot update when recompiled
              end.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% useful cleanup
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

kall() -> 
  exit(whereis(hs),kill),
  exit(whereis(cs),kill),
  exit(whereis(ks),kill).



