
-module(mon1).
-compile([export_all]).

start() ->
    % Spawn the monitor process (which will spawn and link the worker)
    MonitorPid = spawn(fun() -> monitor_worker() end),
    
    % Return MonitorPid for tracking (optional)
    MonitorPid.

monitor_worker() ->
    io:format("Monitor started, will spawn and link worker~n"),
    
    % Set trap_exit flag for the monitor so it can receive exit messages
    process_flag(trap_exit, true),
    
    % Spawn the worker and link it
    WorkerPid = spawn_link(fun() -> worker() end),
    
    io:format("Worker spawned and linked: ~p~n", [WorkerPid]),
    
    % Wait for the worker to crash or exit
    wait_for_worker(WorkerPid).

worker() ->
    io:format("Worker started~n"),
    % Simulate some work
    timer:sleep(4000),
    % Simulate a crash (worker exits)
    io:format("Worker crashing~n"),
    exit(crash).

wait_for_worker(WorkerPid) ->
    receive
        {'DOWN', Pid, Reason, _Info} when Pid == WorkerPid ->
            io:format("Worker crashed with reason: ~p~n", [Reason]),
            % Restart the worker process after crash
            io:format("Restarting worker~n"),
            NewWorkerPid = spawn_link(fun() -> worker() end),
            io:format("New worker spawned and linked: ~p~n", [NewWorkerPid]),
            wait_for_worker(NewWorkerPid);
        {'EXIT', Pid, Reason} when Pid == WorkerPid ->
            io:format("Monitor received exit message: Worker crashed with reason: ~p~n", [Reason]),
            % Restart the worker process after crash
            io:format("Restarting worker~n"),
            NewWorkerPid = spawn_link(fun() -> worker() end),
            io:format("New worker spawned and linked: ~p~n", [NewWorkerPid]),
            wait_for_worker(NewWorkerPid);
        _ ->
            io:format("Monitor received unknown message~n"),
            wait_for_worker(WorkerPid)
    end.

