
-module(mon2).
-compile([export_all]).

start() ->
    % Spawn the monitor process (which will spawn and link the workers)
    MonitorPid = spawn(fun() -> monitor_workers() end),
    
    % Return MonitorPid for tracking (optional)
    MonitorPid.

monitor_workers() ->
    io:format("Monitor started, will spawn and link two workers~n"),
    
    % Set trap_exit flag for the monitor so it can receive exit messages
    process_flag(trap_exit, true),
    
    % Spawn the two worker processes and link them
    Worker1Pid = spawn_link(fun() -> worker(1,4000) end),
    Worker2Pid = spawn_link(fun() -> worker(2,7000) end),
    
    io:format("Worker 1 spawned and linked: ~p~n", [Worker1Pid]),
    io:format("Worker 2 spawned and linked: ~p~n", [Worker2Pid]),
    
    % Wait for either worker to crash or exit
    monLoop([Worker1Pid, Worker2Pid]).

worker(Id,Time) ->
    io:format("Worker ~p started~n", [Id]),
    % Simulate some work
    timer:sleep(Time),
    % Simulate a crash (worker exits)
    io:format("Worker ~p crashing~n", [Id]),
    exit(crash).

monLoop(Pids) ->
    receive
        {'EXIT', Pid, Reason} when Pid =:= hd(Pids) ->
            io:format("Monitor received exit message: Worker ~p crashed with reason: ~p~n", [Pid, Reason]),
            io:format("Restarting Worker ~p~n", [Pid]),
            % Restart the crashed worker
            NewWorkerPid = spawn_link(fun() -> worker(hd(Pids),5000) end),
            io:format("New Worker ~p spawned and linked: ~p~n", [hd(Pids), NewWorkerPid]),
            monLoop(tl(Pids) ++ [NewWorkerPid]);
        
        {'EXIT', Pid, Reason} when Pid =:= hd(tl(Pids)) ->
            io:format("Monitor received exit message: Worker ~p crashed with reason: ~p~n", [Pid, Reason]),
            io:format("Restarting Worker ~p~n", [Pid]),
            % Restart the crashed worker
            NewWorkerPid = spawn_link(fun() -> worker(hd(tl(Pids)),4000) end),
            io:format("New Worker ~p spawned and linked: ~p~n", [hd(tl(Pids)), NewWorkerPid]),
            monLoop(tl(tl(Pids)) ++ [NewWorkerPid]);

        _ ->
            io:format("Monitor received unknown message~n"),
            monLoop(Pids)
    end.

